# Mapping of countries to their corresponding country codes
country_to_code = {
    "US": "US",
    "UK": "UK",
    "Canada": "CA",
    "Australia": "AUS",
    "Germany": "DE",
    "France": "FR",
    "Japan": "JP",
    "China": "CN",
    "India": "IN",
    "Brazil": "BR",
    "Russia": "RU",
    "South Africa": "ZA",
    "Mexico": "MX",
    "Italy": "IT",
    "Spain": "ES",
    "South Korea": "KR",
    "Saudi Arabia": "SA",
    "Argentina": "AR",
    "Turkey": "TR",
    "Netherlands": "NL",
    "Sweden": "SE",
    "Switzerland": "CH",
    "Singapore": "SG",
    "New Zealand": "NZ",
    "Norway": "NO",
    "Denmark": "DK",
    "Ireland": "IE",
    "Finland": "FI",
    "Austria": "AT",
    "Belgium": "BE",
    "Portugal": "PT",
    "Greece": "GR"
    # Add more mappings as needed
}
# Mapping of countries to their corresponding currency codes
country_to_currency = {
    "US": "USD",
    "UK": "GBP",
    "Canada": "CAD",
    "Australia": "AUD",
    "Germany": "EUR",
    "France": "EUR",
    "Japan": "JPY",
    "China": "CNY",
    "India": "INR",
    "Brazil": "BRL",
    "Russia": "RUB",
    "South Africa": "ZAR",
    "Mexico": "MXN",
    "Italy": "EUR",
    "Spain": "EUR",
    "South Korea": "KRW",
    "Saudi Arabia": "SAR",
    "Argentina": "ARS",
    "Turkey": "TRY",
    "Netherlands": "EUR",
    "Sweden": "SEK",
    "Switzerland": "CHF",
    "Singapore": "SGD",
    "New Zealand": "NZD",
    "Norway": "NOK",
    "Denmark": "DKK",
    "Ireland": "EUR",
    "Finland": "EUR",
    "Austria": "EUR",
    "Belgium": "EUR",
    "Portugal": "EUR",
    "Greece": "EUR"
    # Add more country-to-currency mappings as needed
}
# Passport formats for various countries
passport_formats = {
    "US": "AA######",
    "UK": "######",
    "Canada": "AA######",
    "Australia": "########",
    "Germany": "#########",
    "France": "#########",
    "Japan": "AA########",
    "China": "##########",
    "India": "##########",
    "Brazil": "########",
    "Russia": "#########",
    "South Africa": "AA########",
    "Mexico": "########",
    "Italy": "#########",
    "Spain": "#########",
    "South Korea": "########",
    "Saudi Arabia": "AA########",
    "Argentina": "#########",
    "Turkey": "#########",
    "Netherlands": "#########",
    "Sweden": "#########",
    "Switzerland": "#########",
    "Singapore": "AA########",
    "New Zealand": "#########",
    "Norway": "#########",
    "Denmark": "#########",
    "Ireland": "#########",
    "Finland": "#########",
    "Austria": "#########",
    "Belgium": "#########",
    "Portugal": "#########",
    "Greece": "#########"
    # Add more formats for other countries as needed
}
# Social Security Number (SSN) formats for various countries
ssn_formats = {
    "US": "###-##-####",
    "UK": "##-##-##",
    "Canada": "###-###-###",
    "Australia": "#########",
    "Germany": "#########",
    "France": "##-##-##-###",
    "Japan": "###-##-####",
    "China": "##########",
    "India": "##########",
    "Brazil": "###-##-###",
    "Russia": "###-##-####",
    "South Africa": "##-##-##-###",
    "Mexico": "###-##-####",
    "Italy": "#########",
    "Spain": "#########",
    "South Korea": "#########",
    "Saudi Arabia": "##-######-####",
    "Argentina": "##-###-###",
    "Turkey": "#########",
    "Netherlands": "#########",
    "Sweden": "########-####",
    "Switzerland": "###-###-###-###",
    "Singapore": "S########",
    "New Zealand": "#########",
    "Norway": "#########",
    "Denmark": "#########",
    "Ireland": "#########",
    "Finland": "########-###",
    "Austria": "#########",
    "Belgium": "########-###",
    "Portugal": "#########",
    "Greece": "###-##-####"
    # Add more formats for other countries as needed
}
# Phone number formats for various countries
phone_formats = {
    "US": "+1-XXX-XXX-XXXX",
    "UK": "+44-7XXX-XXX-XXX",
    "AUS": "+61-4XX-XXX-XXX",
    "CA": "+1-XXX-XXX-XXXX",  # Canadian format is similar to the US
    "Germany": "+49-XXXX-XXXXXXX",
    "France": "+33-X-XX-XX-XX-XX",
    "Japan": "+81-XX-XXXX-XXXX",
    "China": "+86-1XXX-XXXX-XXXX",
    "India": "+91-9XX-XXXX-XXXX",
    "Brazil": "+55-9XXXX-XXXX",
    "Russia": "+7-XXX-XXX-XX-XX",
    "South Africa": "+27-XX-XXX-XXXX",
    "Mexico": "+52-1-XXX-XXX-XXXX",
    "Italy": "+39-3XX-XXX-XXXX",
    "Spain": "+34-6XX-XXX-XXX",
    "South Korea": "+82-1XXX-XXXX-XXXX",
    "Saudi Arabia": "+966-5XX-XXXX-XXX",
    "Argentina": "+54-9XX-XXXX-XXXX",
    "Turkey": "+90-5XX-XXX-XX-XX",
    "Netherlands": "+31-6-XXXXXXXX",
    "Sweden": "+46-7X-XXX-XX-XX",
    "Switzerland": "+41-7XX-XXX-XXX",
    "Singapore": "+65-9XXX-XXXX",
    "New Zealand": "+64-2X-XXXX-XXXX",
    "Norway": "+47-9X-XX-XX-XX",
    "Denmark": "+45-2X-XX-XX-XX",
    "Ireland": "+353-8X-XXX-XXXX",
    "Finland": "+358-4X-XXXX-XXX",
    "Austria": "+43-6X-XXXXXXX",
    "Belgium": "+32-4X-XX-XX-XX",
    "Portugal": "+351-9XX-XXXXXX",
    "Greece": "+30-6XX-XXX-XXXX"
    # Add more formats for other countries as needed
}
# Mapping of departments to lists of job designations within each department
designation_mapping = {
    "Marketing": [
        "Jr. Executive",
        "Executive",
        "Sr. Executive",
        "Asst. Manager",
        "Manager",
        "Sr. Manager",
        "Asst. Director",
        "Director",
        "AVP",
        "VP",
        "CMO",
    ],
    "Sales": [
        "Sales Associate",
        "Sales Representative",
        "Account Executive",
        "Account Manager",
        "Sales Manager",
        "Sr. Sales Manager",
        "Director of Sales",
        "VP of Sales",
    ],
    "Finance": [
        "Jr. Accountant",
        "Accountant",
        "Sr. Accountant",
        "Asst. Controller",
        "Controller",
        "Sr. Controller",
        "CFO",
    ],
    "IT": [
        "Jr. Developer",
        "Developer",
        "Sr. Developer",
        "Lead Developer",
        "Technical Lead",
        "Manager of IT",
        "Director of IT",
        "CTO",
    ],
    "Operations": [
        "Operations Associate",
        "Operations Coordinator",
        "Operations Supervisor",
        "Operations Manager",
        "Sr. Operations Manager",
        "Director of Operations",
        "VP of Operations",
    ],
    "Procurement": [
        "Procurement Assistant",
        "Procurement Specialist",
        "Procurement Analyst",
        "Procurement Manager",
        "Sr. Procurement Manager",
        "Director of Procurement",
        "VP of Procurement",
    ],
    "Accounts": [
        "Jr. Accountant",
        "Accountant",
        "Sr. Accountant",
        "Asst. Controller",
        "Controller",
        "Sr. Controller",
        "CFO",
    ],
    "HR": [
        "Jr. HR Executive",
        "HR Executive",
        "Sr. HR Executive",
        "HR Manager",
        "Sr. HR Manager",
        "HR Director",
        "VP of HR",
    ],
    "Customer Service": [
        "Customer Service Representative",
        "Customer Service Specialist",
        "Customer Service Coordinator",
        "Customer Service Manager",
        "Sr. Customer Service Manager",
        "Director of Customer Service",
        "VP of Customer Service",
    ],
    "Research and Development": [
        "Research Assistant",
        "Research Associate",
        "Research Scientist",
        "Sr. Research Scientist",
        "Lead Research Scientist",
        "Director of Research and Development",
        "VP of R&D",
    ],
    "Quality Assurance": [
        "QA Tester",
        "QA Engineer",
        "QA Analyst",
        "Sr. QA Analyst",
        "Lead QA Engineer",
        "Director of Quality Assurance",
        "VP of Quality Assurance",
    ],
    "Legal": [
        "Legal Assistant",
        "Legal Secretary",
        "Paralegal",
        "Legal Counsel",
        "Sr. Legal Counsel",
        "Director of Legal",
        "VP of Legal",
    ],
    "Public Relations": [
        "PR Assistant",
        "PR Coordinator",
        "PR Specialist",
        "PR Manager",
        "Sr. PR Manager",
        "Director of PR",
        "VP of PR",
    ],
    "Supply Chain": [
        "Supply Chain Assistant",
        "Supply Chain Coordinator",
        "Supply Chain Analyst",
        "Supply Chain Manager",
        "Sr. Supply Chain Manager",
        "Director of Supply Chain",
        "VP of Supply Chain",
    ],
    "Engineering": [
        "Jr. Engineer",
        "Engineer",
        "Deputy Engineer",
        "Assistant Engineer",
        "Executive Engineer",
        "Superintending Engineer",
        "Asst. Director",
        "Director",
        "AVP",
        "VP",
        "CTO",
    ],
    "Product Management": [
        "Product Assistant",
        "Product Coordinator",
        "Product Manager",
        "Sr. Product Manager",
        "Director of Product Management",
        "VP of Product Management",
    ],
    "Health and Safety": [
        "Safety Officer",
        "Health and Safety Coordinator",
        "Health and Safety Specialist",
        "Health and Safety Manager",
        "Sr. Health and Safety Manager",
        "Director of Health and Safety",
        "VP of Health and Safety",
    ],
    "Training and Development": [
        "Training Assistant",
        "Training Coordinator",
        "Training Specialist",
        "Training Manager",
        "Sr. Training Manager",
        "Director of Training and Development",
        "VP of Training and Development",
    ],
    "Business Development": [
        "Business Development Associate",
        "Business Development Representative",
        "Business Development Executive",
        "Business Development Manager",
        "Sr. Business Development Manager",
        "Director of Business Development",
        "VP of Business Development",
    ],
    "Facilities Management": [
        "Facilities Assistant",
        "Facilities Coordinator",
        "Facilities Manager",
        "Sr. Facilities Manager",
        "Director of Facilities Management",
        "VP of Facilities Management",
    ],
    "Data Science": [
        "Data Scientist",
        "Sr. Data Scientist",
        "Lead Data Scientist",
        "Data Science Manager",
        "Director of Data Science",
        "VP of Data Science",
    ],
}

# Mapping of countries to states and cities within each state
country_to_states_and_cities = {
    "US": {
        "Alabama": [
            "Montgomery",
            "Auburn",
            "Birmingham",
            "Huntsville",
            "Dothan",
            "Mobile",
            "Tuscaloosa",
            "Decatur",
            "Hoover",
            "Florence",
        ],
        "Arizona": [
            "Phoenix",
            "Tucson",
            "Yuma",
            "Tempe",
            "Gilbert",
            "Sedona",
            "Scottsdale",
            "Chandler",
            "Flagstaff",
            "Mesa",
        ],
        "California": [
            "Los Angeles",
            "San Francisco",
            "San Diego",
            "Sacramento",
            "Fresno",
            "San Jose",
            "Oakland",
            "Long Beach",
            "Santa Ana",
            "Anaheim",
        ],
        "Texas": [
            "Houston",
            "Dallas",
            "Austin",
            "San Antonio",
            "Fort Worth",
            "El Paso",
            "Arlington",
            "Corpus Christi",
            "Plano",
            "Laredo",
        ],
        "New York": [
            "New York City",
            "Buffalo",
            "Rochester",
            "Albany",
            "Syracuse",
            "Yonkers",
            "White Plains",
            "Utica",
            "Schenectady",
            "New Rochelle",
        ],
        "Florida": [
            "Miami",
            "Orlando",
            "Tampa",
            "Jacksonville",
            "Fort Lauderdale",
            "Pensacola",
            "Tallahassee",
            "St. Petersburg",
            "Naples",
            "Fort Myers",
        ],
        "Illinois": [
            "Chicago",
            "Springfield",
            "Peoria",
            "Aurora",
            "Rockford",
            "Joliet",
            "Naperville",
            "Elgin",
            "Waukegan",
            "Champaign",
        ],
        "Georgia": [
            "Atlanta",
            "Augusta",
            "Savannah",
            "Columbus",
            "Macon",
            "Athens",
            "Albany",
            "Sandy Springs",
            "Roswell",
            "Johns Creek",
        ],
        "Michigan": [
            "Detroit",
            "Grand Rapids",
            "Ann Arbor",
            "Lansing",
            "Flint",
            "Kalamazoo",
            "Dearborn",
            "Sterling Heights",
            "Livonia",
            "Troy",
        ],
        "Colorado": [
            "Denver",
            "Colorado Springs",
            "Aurora",
            "Fort Collins",
            "Lakewood",
            "Thornton",
            "Arvada",
            "Westminster",
            "Centennial",
            "Boulder",
        ],
    },
    "UK": {
        "Norfolk": ["Holt", "Cromer", "King's Lynn", "Aylsham", "Hunstanton"],
        "Kent": ["Canterbury", "Maidstone", "Dover", "Margate", "Ashford"],
        "Surrey": ["Guildford", "Woking", "Camberley", "Epsom", "Esher"],
        "Essex": ["Chelmsford", "Basildon", "Colchester", "Southend-on-Sea", "Harlow"],
        "Hampshire": [
            "Winchester",
            "Southampton",
            "Portsmouth",
            "Basingstoke",
            "Gosport",
        ],
        "West Midlands": [
            "Birmingham",
            "Coventry",
            "Wolverhampton",
            "Solihull",
            "Dudley",
        ],
        "Lancashire": ["Manchester", "Liverpool", "Preston", "Blackpool", "Burnley"],
        "Devon": ["Exeter", "Plymouth", "Torquay", "Paignton", "Exmouth"],
        "Oxfordshire": ["Oxford", "Banbury", "Bicester", "Abingdon", "Witney"],
        "North Yorkshire": [
            "York",
            "Harrogate",
            "Scarborough",
            "Middlesbrough",
            "Selby",
        ],
    },
    "Canada": {
        "Ontario": [
            "Toronto",
            "Ottawa",
            "Hamilton",
            "London",
            "Mississauga",
            "Brampton",
            "Markham",
            "Vaughan",
            "Windsor",
            "Kitchener",
        ],
        "Quebec": [
            "Montreal",
            "Quebec City",
            "Laval",
            "Gatineau",
            "Longueuil",
            "Sherbrooke",
            "Saguenay",
            "Levis",
            "Trois-Rivieres",
            "Terrebonne",
        ],
        "British Columbia": [
            "Vancouver",
            "Surrey",
            "Burnaby",
            "Richmond",
            "Kelowna",
            "Kamloops",
            "Nanaimo",
            "Victoria",
            "Abbotsford",
            "Langley",
        ],
        "Alberta": [
            "Calgary",
            "Edmonton",
            "Red Deer",
            "Lethbridge",
            "St. Albert",
            "Medicine Hat",
            "Grande Prairie",
            "Airdrie",
            "Spruce Grove",
            "Leduc",
        ],
        "Manitoba": [
            "Winnipeg",
            "Brandon",
            "Steinbach",
            "Thompson",
            "Portage la Prairie",
            "Selkirk",
            "Winkler",
            "Dauphin",
            "Morden",
            "Flin Flon",
        ],
    },
    "Australia": {
        "New South Wales": [
            "Sydney",
            "Newcastle",
            "Central Coast",
            "Wollongong",
            "Maitland",
            "Blue Mountains",
            "Queanbeyan",
            "Goulburn",
            "Tamworth",
            "Orange",
        ],
        "Victoria": [
            "Melbourne",
            "Geelong",
            "Ballarat",
            "Bendigo",
            "Shepparton",
            "Melton",
            "Mildura",
            "Wodonga",
            "Warrnambool",
            "Sunbury",
        ],
        "Queensland": [
            "Brisbane",
            "Gold Coast",
            "Sunshine Coast",
            "Townsville",
            "Cairns",
            "Toowoomba",
            "Mackay",
            "Rockhampton",
            "Bundaberg",
            "Hervey Bay",
        ],
        "Western Australia": [
            "Perth",
            "Mandurah",
            "Bunbury",
            "Geraldton",
            "Albany",
            "Kalgoorlie",
            "Broome",
            "Karratha",
            "Port Hedland",
            "Rockingham",
        ],
        "South Australia": [
            "Adelaide",
            "Mount Gambier",
            "Whyalla",
            "Murray Bridge",
            "Port Augusta",
            "Port Pirie",
            "Port Lincoln",
            "Victor Harbor",
            "Gawler",
            "Goolwa",
        ],
    },
}
# Mapping of product categories to lists of product names within each category
product_names_dict = {
    "Electronics": [
        "Smartphone",
        "Laptop",
        "Headphones",
        "Tablet",
        "Camera",
        "Smartwatch",
        "Speaker",
        "Gaming Console",
        "Wireless Earbuds",
        "Power Bank",
        "TV",
        "Fitness Tracker",
        "Virtual Reality Headset",
        "Printer",
        "External Hard Drive",
        "Drone",
        "Bluetooth Keyboard",
        "Home Security Camera",
        "Wireless Charger",
        "Portable Projector",
    ],
    "Clothing": [
        "T-shirt",
        "Jeans",
        "Dress",
        "Shoes",
        "Sweater",
        "Jacket",
        "Hat",
        "Socks",
        "Skirt",
        "Pants",
        "Scarf",
        "Gloves",
        "Hoodie",
        "Boots",
        "Watch",
        "Tie",
        "Belt",
        "Sunglasses",
        "Backpack",
        "Umbrella",
    ],
    "Home and Kitchen": [
        "Coffee Maker",
        "Blender",
        "Toaster",
        "Microwave",
        "Pots and Pans",
        "Cutlery Set",
        "Dishwasher",
        "Air Fryer",
        "Refrigerator",
        "Kettle",
        "Vacuum Cleaner",
        "Food Processor",
        "Stand Mixer",
        "Bread Maker",
        "Waffle Maker",
        "Slow Cooker",
        "Knife Set",
        "Coffee Grinder",
        "Juicer",
        "Espresso Machine",
    ],
    "Books": [
        "Fiction",
        "Non-Fiction",
        "Mystery",
        "Science Fiction",
        "Biography",
        "Fantasy",
        "Romance",
        "Thriller",
        "Self-Help",
        "Cookbook",
        "History",
        "Science",
        "Art",
        "Travel",
        "Poetry",
        "Business",
        "Health",
        "Children's",
        "Classic Literature",
        "Humor",
    ],
    "Toys": [
        "Action Figures",
        "Dolls",
        "Building Blocks",
        "Board Games",
        "Puzzles",
        "Remote Control Cars",
        "Stuffed Animals",
        "Educational Toys",
        "Outdoor Toys",
        "Art Supplies",
        "Sports Equipment",
        "Play Kitchen",
        "Toy Cars",
        "Dress-Up Clothes",
        "Musical Toys",
        "Science Kits",
        "Robot Toys",
        "Magic Sets",
        "Water Guns",
        "Toy Trains",
    ],
    "Beauty and Personal Care": [
        "Shampoo",
        "Conditioner",
        "Body Wash",
        "Face Moisturizer",
        "Sunscreen",
        "Makeup Remover",
        "Lip Balm",
        "Perfume",
        "Hair Dryer",
        "Flat Iron",
        "Electric Toothbrush",
        "Razor",
        "Cologne",
        "Nail Polish",
        "Facial Cleanser",
        "Hand Cream",
        "Deodorant",
        "Hair Styling Gel",
        "Curling Iron",
        "Eye Cream",
    ],
    "Sports and Outdoors": [
        "Tent",
        "Hiking Boots",
        "Backpack",
        "Camping Stove",
        "Sleeping Bag",
        "Binoculars",
        "Fishing Rod",
        "Bicycle",
        "Running Shoes",
        "Water Bottle",
        "Golf Clubs",
        "Ski Gear",
        "Yoga Mat",
        "Swimming Goggles",
        "Soccer Ball",
        "Basketball",
        "Tennis Racket",
        "Football",
        "Baseball Glove",
        "Hammock",
    ],
    "Automotive": [
        "Car Battery",
        "Tire Pressure Gauge",
        "Car Wax",
        "Oil Filter",
        "Windshield Wipers",
        "Jump Starter",
        "Car Seat Covers",
        "Car Vacuum",
        "Dash Cam",
        "GPS Navigation System",
        "Car Air Freshener",
        "Roof Rack",
        "Car Charger",
        "Car Jack",
        "Floor Mats",
        "Car Wash Kit",
        "Tool Set",
        "Snow Chains",
        "Car Cover",
        "Bluetooth Car Adapter",
    ],
    "Health and Household": [
        "Vitamins",
        "First Aid Kit",
        "Air Purifier",
        "Humidifier",
        "Water Filter",
        "Thermometer",
        "Weight Scale",
        "Blood Pressure Monitor",
        "Sleeping Mask",
        "Electric Blanket",
        "Essential Oils",
        "Cleaning Supplies",
        "Laundry Detergent",
        "Trash Bags",
        "Paper Towels",
        "Hand Sanitizer",
        "Bath Towels",
        "Mop and Bucket",
        "Dish Soap",
        "Toothpaste",
    ],
    "Tools and Home Improvement": [
        "Drill",
        "Screwdriver Set",
        "Wrench",
        "Toolbox",
        "Power Drill",
        "Circular Saw",
        "Measuring Tape",
        "Paint Brushes",
        "Ladder",
        "Safety Glasses",
        "Socket Set",
        "Hacksaw",
        "Plunger",
        "Extension Cord",
        "Cordless Screwdriver",
        "Flashlight",
        "Nails and Screws Assortment",
        "Level",
        "Stud Finder",
        "Work Gloves",
    ],
    "Grocery": [
        "Cereal",
        "Pasta",
        "Rice",
        "Canned Soup",
        "Frozen Vegetables",
        "Fresh Fruits",
        "Bread",
        "Milk",
        "Eggs",
        "Cheese",
        "Yogurt",
        "Snack Bars",
        "Juice",
        "Coffee",
        "Tea",
        "Cookies",
        "Chocolate",
        "Condiments",
        "Cereal Bars",
        "Nuts",
    ],
    "Pet Supplies": [
        "Dog Food",
        "Cat Food",
        "Pet Bed",
        "Dog Collar",
        "Cat Litter",
        "Fish Tank",
        "Bird Cage",
        "Pet Toys",
        "Dog Leash",
        "Pet Grooming Kit",
        "Reptile Habitat",
        "Small Animal Cage",
        "Aquarium Filter",
        "Hamster Wheel",
        "Pet Carrier",
        "Bird Feeder",
        "Dog Shampoo",
        "Cat Scratcher",
        "Fish Food",
        "Treats",
    ],
    "Baby": [
        "Diapers",
        "Baby Wipes",
        "Baby Formula",
        "Baby Clothes",
        "Pacifiers",
        "Baby Bottles",
        "Stroller",
        "Car Seat",
        "Baby Monitor",
        "High Chair",
        "Crib",
        "Changing Table",
        "Baby Carrier",
        "Playpen",
        "Diaper Bag",
        "Baby Shoes",
        "Teething Toys",
        "Baby Blanket",
        "Nursery Decor",
        "Baby Swaddle",
    ],
    "Office Products": [
        "Desk Organizer",
        "Laptop Stand",
        "Office Chair",
        "Printer Paper",
        "Notebooks",
        "Pens",
        "Stapler",
        "Desk Lamp",
        "Filing Cabinet",
        "Whiteboard",
        "Mouse Pad",
        "Computer Monitor",
        "Desk Calendar",
        "Headset",
        "Desk Fan",
        "Hole Punch",
        "Scissors",
        "Paper Clips",
        "Business Cards Holder",
        "Desk Clock",
    ],
    "Jewelry": [
        "Necklace",
        "Bracelet",
        "Earrings",
        "Ring",
        "Watch",
        "Anklet",
        "Brooch",
        "Cufflinks",
        "Pendant",
        "Charm Bracelet",
        "Jewelry Box",
        "Hairpin",
        "Bangle",
        "Pearl Necklace",
        "Diamond Stud Earrings",
        "Gold Chain",
        "Gemstone Ring",
        "Silver Hoop Earrings",
        "Wedding Band",
        "Lockets",
    ],
    "Movies and TV": [
        "DVDs",
        "Blu-ray Discs",
        "TV Series Box Set",
        "Streaming Device",
        "Movie Posters",
        "Popcorn Maker",
        "Movie Tickets",
        "Soundbar",
        "Projector",
        "Home Theater System",
        "TV Stand",
        "Media Storage Tower",
        "Movie Merchandise",
        "Collectibles",
        "Movie Trivia Games",
        "Movie Soundtracks",
        "Film Cameras",
        "Action Figure Collectibles",
        "Movie Scripts",
        "Movie Magazines",
    ],
    "Video Games": [
        "Gaming Mouse",
        "Gaming Keyboard",
        "Gaming Headset",
        "Gaming Chair",
        "Gaming Console",
        "Video Game Controller",
        "Gaming Laptop",
        "Gaming Desk",
        "Gaming Monitor",
        "Gaming Mouse Pad",
        "Virtual Reality Gaming System",
        "Gaming Microphone",
        "Gaming Glasses",
        "Gaming Accessories Kit",
        "Gaming Storage Tower",
        "Video Games Collection",
        "Gaming Capture Card",
        "Gaming Speakers",
        "Gaming Router",
        "Gaming Merchandise",
    ],
    "Musical Instruments": [
        "Guitar",
        "Keyboard",
        "Drum Set",
        "Violin",
        "Saxophone",
        "Trumpet",
        "Flute",
        "Piano",
        "Acoustic Guitar",
        "Electric Bass",
        "Clarinet",
        "Cello",
        "Mandolin",
        "Banjo",
        "Harp",
        "Accordion",
        "Ukulele",
        "Djembe",
        "Synthesizer",
        "Microphone",
    ],
    "Industrial and Scientific": [
        "Lab Equipment",
        "Microscope",
        "Centrifuge",
        "Chemistry Set",
        "Safety Goggles",
        "Laboratory Glassware",
        "Industrial Scale",
        "Lab Coats",
        "Safety Gloves",
        "Safety Boots",
        "Calipers",
        "Thermocouples",
        "Industrial Fans",
        "Soldering Iron",
        "Digital Multimeter",
        "Bunsen Burner",
        "Electronic Components",
        "Laboratory Refrigerator",
        "Fume Hood",
        "Lab Bench",
    ],
    "Arts, Crafts, and Sewing": [
        "Paint Set",
        "Sketchbooks",
        "Canvas",
        "Watercolor Brushes",
        "Sewing Machine",
        "Knitting Needles",
        "Crochet Hooks",
        "Quilting Fabric",
        "Scissors",
        "Art Easels",
        "Acrylic Paints",
        "Oil Paints",
        "Colored Pencils",
        "Craft Glue",
        "Craft Paper",
        "Embroidery Thread",
        "Beads and Jewelry Making Kits",
        "Fabric Markers",
        "Sewing Patterns",
        "Art Apron",
    ],
}
# List of common male first names
male_names = [
    "John",
    "Robert",
    "Michael",
    "William",
    "David",
    "Richard",
    "Joseph",
    "Charles",
    "Thomas",
    "Christopher",
    "Daniel",
    "Paul",
    "Mark",
    "Donald",
    "George",
    "Kenneth",
    "Steven",
    "Edward",
    "Brian",
    "Ronald",
    "Anthony",
    "Kevin",
    "Jason",
    "Matthew",
    "Gary",
    "Timothy",
    "Jose",
    "Larry",
    "Jeffrey",
    "Frank",
    "Scott",
    "Eric",
    "Stephen",
    "Andrew",
    "Raymond",
    "Gregory",
    "Joshua",
    "Jerry",
    "Dennis",
    "Walter",
    "Patrick",
    "Peter",
    "Harold",
    "Douglas",
    "Henry",
    "Carl",
    "Arthur",
    "Ryan",
    "Roger",
    "Joe",
    "Juan",
    "Jack",
    "Albert",
    "Jonathan",
    "Justin",
    "Terry",
    "Gerald",
    "Keith",
    "Samuel",
    "Willie",
    "Ralph",
    "Lawrence",
    "Nicholas",
    "Roy",
    "Benjamin",
    "Bruce",
    "Brandon",
    "Adam",
    "Harry",
    "Fred",
    "Wayne",
    "Billy",
    "Steve",
    "Louis",
    "Jeremy",
    "Aaron",
    "Randy",
    "Howard",
    "Eugene",
    "Carlos",
    "Russell",
    "Bobby",
    "Victor",
    "Martin",
    "Ernest",
    "Phillip",
    "Todd",
    "Jesse",
    "Craig",
    "Alan",
    "Shawn",
    "Clarence",
    "Sean",
    "Philip",
    "Chris",
    "Johnny",
    "Earl",
    "Jimmy",
    "Antonio",
    "Danny",
]
# List of common female first names
female_names = [
    "Mary",
    "Patricia",
    "Jennifer",
    "Linda",
    "Elizabeth",
    "Barbara",
    "Susan",
    "Jessica",
    "Sarah",
    "Karen",
    "Nancy",
    "Lisa",
    "Margaret",
    "Betty",
    "Dorothy",
    "Sandra",
    "Ashley",
    "Kimberly",
    "Donna",
    "Emily",
    "Michelle",
    "Carol",
    "Amanda",
    "Melissa",
    "Deborah",
    "Stephanie",
    "Rebecca",
    "Laura",
    "Sharon",
    "Cynthia",
    "Kathleen",
    "Helen",
    "Amy",
    "Shirley",
    "Angela",
    "Frances",
    "Anna",
    "Ruth",
    "Virginia",
    "Brenda",
    "Pamela",
    "Catherine",
    "Debra",
    "Janet",
    "Carolyn",
    "Renee",
    "Kathryn",
    "Lori",
    "Tiffany",
    "Connie",
    "Victoria",
    "Evelyn",
    "Grace",
    "Diane",
    "Alice",
    "Julie",
    "Heather",
    "Teresa",
    "Doris",
    "Gloria",
    "Judy",
    "Mildred",
    "Denise",
    "Jane",
    "Martha",
    "Andrea",
    "Sara",
    "Jacqueline",
    "Ann",
    "Wanda",
    "Bonnie",
    "Julia",
    "Ruby",
    "Lois",
    "Tina",
    "Phyllis",
    "Norma",
    "Paula",
    "Diana",
    "Annie",
    "Lillian",
    "Emily",
    "Robin",
    "Peggy",
    "Crystal",
    "Gladys",
    "Rita",
    "Dawn",
    "Connie",
    "Fannie",
    "Holly",
    "Erin",
    "April",
    "Kristen",
    "Pauline",
]
# List of common last names
last_names = [
    "Smith",
    "Johnson",
    "Brown",
    "Davis",
    "Wilson",
    "Moore",
    "Taylor",
    "Anderson",
    "Thomas",
    "Jackson",
    "White",
    "Harris",
    "Martin",
    "Thompson",
    "Garcia",
    "Martinez",
    "Robinson",
    "Clark",
    "Rodriguez",
    "Lewis",
    "Lee",
    "Walker",
    "Hall",
    "Allen",
    "Young",
    "Hernandez",
    "King",
    "Wright",
    "Lopez",
    "Hill",
    "Scott",
    "Green",
    "Adams",
    "Baker",
    "Nelson",
    "Carter",
    "Mitchell",
    "Perez",
    "Roberts",
    "Turner",
    "Phillips",
    "Campbell",
    "Parker",
    "Evans",
    "Edwards",
    "Collins",
    "Stewart",
    "Sanchez",
    "Morris",
    "Rogers",
    "Reed",
    "Cook",
    "Morgan",
    "Bell",
    "Murphy",
    "Bailey",
    "Rivera",
    "Cooper",
    "Richardson",
    "Cox",
    "Howard",
    "Ward",
    "Torres",
    "Peterson",
    "Gray",
    "Ramirez",
    "James",
    "Watson",
    "Brooks",
    "Kelly",
    "Sanders",
    "Price",
    "Bennett",
    "Wood",
    "Barnes",
    "Ross",
    "Henderson",
    "Coleman",
    "Jenkins",
    "Perry",
    "Powell",
    "Long",
    "Patterson",
    "Hughes",
    "Flores",
    "Washington",
    "Butler",
    "Simmons",
    "Foster",
    "Gonzales",
    "Bryant",
    "Alexander",
    "Russell",
    "Griffin",
    "Diaz",
    "Hayes",
    "Myers",
    "Ford",
    "Hamilton",
    "Graham",
    "Sullivan",
    "Wallace",
]
# Dictionary mapping country codes to latitude ranges
country_latitudes = {
    "US": (24.396308, 49.384358),  # Contiguous United States
    "UK": (49.91224, 58.635),  # United Kingdom
    "Canada": (41.681982, 83.110626),  # Canada
    "Australia": (-43.634597, -10.668185),  # Australia
    "Germany": (47.270111, 55.056482),
    "France": (41.303, 51.124),
    "Japan": (24.396308, 45.551483),
    "China": (18.159073, 53.560815),
    "India": (8.076, 37.084),
    "Brazil": (-33.751667, 5.272222),
    "Russia": (41.198476, 81.858),
    "South Africa": (-34.834019, -17.589844),
    "Mexico": (14.534472, 32.718711),
    "Italy": (35.813081, 47.092),  # Approximate mainland Italy
    "Spain": (27.498, 43.791),
    "South Korea": (33.19, 38.625),
    "Saudi Arabia": (16.347, 32.159),
    "Argentina": (-55.051667, -21.781111),
    "Turkey": (35.8131, 42.107),
    "Netherlands": (50.77083, 53.359167),
    "Sweden": (55.34, 69.07),
    "Switzerland": (45.818, 47.81),
    "Singapore": (1.258, 1.478),
    "New Zealand": (-47.3, -33.6),
    "Norway": (57.885, 71.19),
    "Denmark": (54.57, 57.74),
    "Ireland": (51.427, 55.385),
    "Finland": (59.808, 70.095),
    "Austria": (46.372, 49.012),
    "Belgium": (49.497, 51.505),
    "Portugal": (36.959, 42.141),
    "Greece": (35.81, 41.748)
    # Add more countries and latitude ranges as needed
}
# Dictionary mapping country codes to longitude ranges
country_longitudes = {
    "US": (-125.000000, -66.934570),  # Contiguous United States
    "UK": (-8.646362, 1.763306),  # United Kingdom
    "Canada": (-141.000000, -52.617390),  # Canada
    "Australia": (112.921112, 153.6349),  # Australia
    "Germany": (5.988659, 15.016995),
    "France": (-5.453, 9.561),
    "Japan": (122.934570, 153.986672),
    "China": (73.502519, 135.000000),
    "India": (68.186, 97.416),
    "Brazil": (-73.982226, -34.729444),
    "Russia": (-168.534159, 169.039551),
    "South Africa": (16.344452, 32.891960),
    "Mexico": (-117.12771, -86.590573),
    "Italy": (6.6193, 18.540),  # Approximate mainland Italy
    "Spain": (-18.167, 4.327),
    "South Korea": (125.887, 129.583),
    "Saudi Arabia": (34.495, 55.668),
    "Argentina": (-73.582778, -53.637222),
    "Turkey": (25.813, 44.79),
    "Netherlands": (3.358333, 7.210833),
    "Sweden": (11.11, 24.17),
    "Switzerland": (5.957, 10.491),
    "Singapore": (103.638, 104.072),
    "New Zealand": (166.464, 178.841),
    "Norway": (4.992, 31.03),
    "Denmark": (7.43, 15.12),
    "Ireland": (-10.477, -5.414),
    "Finland": (20.645, 31.586),
    "Austria": (9.535, 17.166),
    "Belgium": (2.54, 6.408),
    "Portugal": (-9.492, -6.19),
    "Greece": (20.869, 29.65)
    # Add more countries and longitude ranges as needed
}
# List of company name prefixes
company_prefix = [
    "secure",
    "premium",
    "delux",
    "innovative",
    "creative",
    "techno",
    "global",
    "dynamic",
    "synergistic",
    "stellar",
    "exquisite",
    "elite",
    "ultimate",
    "visionary",
    "pinnacle",
    "vibrant",
    "profound",
    "paragon",
    "zenith",
    "prime",
    "stellar",
    "supreme",
    "elegant",
    "quantum",
    "infinity",
    "excellent",
    "majestic",
    "efficiency",
    "harmony",
    "splendid",
    "superior",
    "inspire",
    "insight",
    "integral",
    "luminous",
    "fusion",
    "radiant",
    "evolve",
    "accelerate",
    "innate",
    "innovation",
    "excellence",
    "embrace",
    "prosperity",
    "forge",
    "synergy",
    "catalyst",
    "spectrum",
    "horizon",
    "unison",
    "integrity",
    "precision",
    "transcend",
    "illuminate",
    "noble",
    "zen",
    "velocity",
    "synergy",
    "grandeur",
    "genesis",
    "effulgent",
    "harmonic",
    "quantum",
    "resonance",
    "stellar",
    "invincible",
    "intrepid",
    "innate",
    "stellar",
    "supreme",
    "eclipse",
    "rhapsody",
    "vanguard",
    "purity",
    "virtue",
    "equilibrium",
    "evolve",
    "integrity",
    "luminous",
    "triumph",
    "prosper",
    "zenith",
    "synergy",
    "vivid",
    "integral",
]
# List of company name suffixes
company_suffix = [
    "tech",
    "metal",
    "chemical",
    "services",
    "solutions",
    "pharmaceuticals",
    "machineries",
    "automotive",
    "glass",
    "woodworks",
    "systems",
    "networks",
    "solutions",
    "enterprises",
    "industries",
    "ventures",
    "technologies",
    "dynamics",
    "concepts",
    "innovations",
    "consulting",
    "labs",
    "group",
    "mechanics",
]
# List of words for generating email addresses (email_1)
email_1 = [
    "beautiful",
    "opera",
    "sunshine",
    "desert",
    "rock",
    "rhythm",
    "ocean",
    "tulip",
    "norwester",
    "mew",
    "love",
]
# List of words for generating email addresses (email_2)
email_2 = [
    "xoxo",
    "story",
    "poetry",
    "water",
    "fire",
    "abc",
    "findme",
    "lighthouse",
    "glasswall",
    "silk",
    "heaven",
]
# List of words for generating email addresses (email_3)
email_3 = ["a2z", "1010", "1996", "mozzarella", "x", "80s", "work", "uni"]
# List of default department names
default_departments = [
    "Marketing",
    "Sales",
    "Finance",
    "IT",
    "Operations",
    "Procurement",
    "Accounts",
    "HR",
    "Customer Service",
    "Research and Development",
    "Quality Assurance",
    "Legal",
    "Public Relations",
    "Supply Chain",
    "Engineering",
    "Product Management",
    "Health and Safety",
    "Training and Development",
    "Business Development",
    "Facilities Management",
    "Data Science",
]

# List of common words for generating passwords (password_words_1)
password_words_1 = [
    "apple",
    "david",
    "wowza",
    "happy",
    "banana",
    "chocolate",
    "abcdef",
    "asdfgh",
    "admin",
    "diamond",
    "eagle",
]
# List of common words and characters for generating passwords (password_words_2)
password_words_2 = [
    "123",
    "456",
    "password",
    "123456",
    "secret",
    "pass",
    "!",
    "@",
    "hello",
    "000",
    "000000",
    "sunshine",
]
# List of default industry categories
default_industry = [
    "Fashion and Retail",
    "Telecom",
    "Technology",
    "Automotive",
    "Media and Entertainment",
    "Healthcare",
    "Finance and Banking",
    "Energy and Utilities",
    "Hospitality and Tourism",
    "Food and Beverage",
    "Real Estate",
    "Education",
    "Aerospace",
    "Environmental",
    "Biotechnology",
    "Pharmaceuticals",
    "Logistics and Transportation",
    "Sports and Recreation",
    "Insurance",
    "Government and Public Services",
    "Consumer Goods",
    "Manufacturing",
    "Agriculture",
    "Information Technology",
    "E-commerce",
    "Consulting",
    "Marketing and Advertising",
    "Construction",
    "Nonprofit and Social Services",
    "Telecommunications",
    "Gaming",
    "Research and Development",
    "Space Exploration",
    "Fitness and Wellness",
    "Legal Services",
    "Art and Design",
    "Renewable Energy",
    "Robotics",
    "Artificial Intelligence",
    "Virtual Reality",
    "Augmented Reality",
    "Cybersecurity",
    "Blockchain",
    "Quantum Computing",
]
# List of default product categories
default_product_categories = [
    "Electronics",
    "Clothing",
    "Home and Kitchen",
    "Books",
    "Toys",
    "Beauty and Personal Care",
    "Sports and Outdoors",
    "Automotive",
    "Health and Household",
    "Tools and Home Improvement",
    "Grocery",
    "Pet Supplies",
    "Baby",
    "Office Products",
    "Jewelry",
    "Movies and TV",
    "Video Games",
    "Musical Instruments",
    "Industrial and Scientific",
    "Arts, Crafts, and Sewing",
]



import random
from datetime import datetime, timedelta
import string
from itertools import count

from ficto.helper.data_module import *


# Function to generate a random first name
def generate_first_name(gender=None, male_names=male_names, female_names=female_names):
    """
    Generate a random first name.

    Parameters:
    - gender (str, optional): Specify the gender for the generated name ("male" or "female").
    - male_names (list, optional): List of male first names. Default is None.
    - female_names (list, optional): List of female first names. Default is None.

    Returns:
    - str: A randomly generated first name based on the specified gender.
    """
    if gender == "male":
        return random.choice(male_names)
    elif gender == "female":
        return random.choice(female_names)
    else:
        return random.choice(male_names + female_names)


# Function to generate a random last name
def generate_last_name(last_names=last_names):
    """
    Generate a random last name.

    Parameters:
    - last_names (list, optional): List of last names. Default is None.

    Returns:
    - str: A randomly generated last name.
    """
    return random.choice(last_names)


# Function to generate a random gender
def generate_gender(first_name=None, male_names=male_names, female_names=female_names):
    """
    Generate a random gender based on the provided first name or randomly if no name is provided.

    Parameters:
    - first_name (str, optional): First name to determine the gender. Default is None.
    - male_names (list, optional): List of male first names. Default is None.
    - female_names (list, optional): List of female first names. Default is None.

    Returns:
    - str: A randomly generated gender ("Male" or "Female").
    """
    male_names = set(male_names)
    female_names = set(female_names)

    # If a first name is provided and recognized
    if first_name:
        if first_name in male_names:
            return "Male"
        else:
            return "Female"

    # If no first name is provided or if the name is not recognized
    return random.choice(["Male", "Female"])


# Function to generate a random currency based on country
def generate_currency(country=None):
    """
    Generate a random currency code based on the provided country.

    Parameters:
    - country (str, optional): Country code. Default is None.

    Returns:
    - str: Randomly generated currency code. Defaults to "USD" if no country is specified.
    """
    if country:
        return country_to_currency.get(country, "USD")
    else:
        return "USD"


# Function to generate a random phone number
def generate_phone_number(country_code="US", phone_formats=phone_formats):
    """
    Generate a random phone number based on the specified country code and phone number format.

    Parameters:
    - country_code (str, optional): Country code for the phone number. Default is "US".
    - phone_formats (dict, optional): Dictionary containing phone number formats for different countries. Default is phone_formats.

    Returns:
    - str: Randomly generated phone number.
    """
    format_string = phone_formats.get(country_code, phone_formats["US"])
    # Replace each 'X' with a random digit
    phone_number = "".join(
        random.choice("0123456789") if c == "X" else c for c in format_string
    )

    return phone_number


# Function to generate a random date
def generate_random_date(start="01/01/1900", end="31/12/2023"):
    """
    Generate a random date within the specified date range.

    Parameters:
    - start (str, optional): Start date in the format "DD/MM/YYYY". Default is "01/01/1900".
    - end (str, optional): End date in the format "DD/MM/YYYY". Default is "31/12/2023".

    Returns:
    - datetime: Randomly generated date.
    """
    start = datetime.strptime(start, "%d/%m/%Y")
    end = datetime.strptime(end, "%d/%m/%Y")
    delta = end - start
    random_days = random.randrange(delta.days)
    return start + timedelta(days=random_days)


# Function to generate a random integer between start and end (inclusive)
def generate_customint(start=0, end=100, round_digits=1):
    """
    Generate a random integer within the specified range.

    Parameters:
    - start (int, optional): Start of the range (inclusive). Default is 0.
    - end (int, optional): End of the range (inclusive). Default is 100.
    - round_digits (int, optional): Number of digits to round to. Default is 1.

    Returns:
    - int: Randomly generated integer.
    """
    value = random.randint(start, end)
    return round(value / round_digits) * round_digits


# Function to generate a random float between start and end (inclusive)
def generate_customfloat(start=0.0, end=100.0, round_digits=2):
    """
    Generate a random float within the specified range.

    Parameters:
    - start (float, optional): Start of the range (inclusive). Default is 0.0.
    - end (float, optional): End of the range (inclusive). Default is 100.0.
    - round_digits (int, optional): Number of digits to round to. Default is 2.

    Returns:
    - float: Randomly generated float.
    """
    value = random.uniform(start, end)
    return round(value, round_digits)


# Function to generate a random string from a list of values
def generate_customlist(values):
    """
    Generate a random string from a given list of values.

    Parameters:
    - values (list): List of values to choose from.

    Returns:
    - str: Randomly chosen value from the list.
    """
    return random.choice(values)


# Function to generate a random unique ID based on character limit and numeric flag
def generate_customuuid(characterlimit=10, numeric=False):
    """
    Generate a random unique ID based on the specified character limit and numeric flag.

    Parameters:
    - characterlimit (int, optional): Maximum length of the generated ID. Default is 10.
    - numeric (bool, optional): If True, generate a numeric ID. Default is False.

    Returns:
    - str: Randomly generated unique ID.
    """
    if numeric:
        return "".join(random.choice(string.digits) for _ in range(characterlimit))
    else:
        return "".join(
            random.choice(string.ascii_letters + string.digits)
            for _ in range(characterlimit)
        )


# Function to generate a random "lorem ipsum" text up to the character limit
def generate_customlorem(characterlimit=200):
    """
    Generate a random "lorem ipsum" text up to the specified character limit.

    Parameters:
    - characterlimit (int, optional): Maximum length of the generated text. Default is 200.

    Returns:
    - str: Randomly generated "lorem ipsum" text.
    """
    lorem_ipsum = (
        "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "
        "Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. "
        "Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. "
        "Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
    )
    startchar = random.randint(0, characterlimit)
    lorem_ipsum = lorem_ipsum[startchar : (startchar + characterlimit)]
    # Capitalize the first letter
    lorem_ipsum = lorem_ipsum[0].capitalize() + lorem_ipsum[1:]
    return lorem_ipsum


# Function to generate a random email address using meaningful words
def generate_email(
    emailproviders=["gmail.com", "yahoo.com", "hotmail.com", "live.com", "outlook.com"]
):
    """
    Generate a random email address using meaningful words and a randomly selected email provider.

    Parameters:
    - emailproviders (list, optional): List of email providers. Default is ["gmail.com", "yahoo.com", "hotmail.com", "live.com", "outlook.com"].

    Returns:
    - str: Randomly generated email address.
    """
    # Randomly select an email provider from the list
    provider = random.choice(emailproviders)

    # Randomly select one meaningful word from each list
    word1 = random.choice(email_1)
    word2 = random.choice(email_2)
    word3 = random.choice(email_3)
    newlist = [word1, word2, word3]
    word4 = random.choice(newlist)
    word5 = random.choice(newlist)
    while word5 == word4:
        word5 = random.choice(newlist)

    # Combine the words and email provider to create the email address
    email = f"{word4}{word5}@{provider}"

    return email


# Function to generate a random web URL with a meaningful word and domain
def generate_website(domain=[".com"]):
    """
    Generate a random web URL with a meaningful word, domain, and 'www.'.

    Parameters:
    - domain (list, optional): List of domain extensions. Default is [".com"].

    Returns:
    - str: Randomly generated web URL.
    """
    # Randomly select a meaningful word from the prefix and suffix lists
    selected_prefix = random.choice(company_prefix)
    selected_suffix = random.choice(company_suffix)

    # Randomly select a domain from the list
    selected_domain = random.choice(domain)

    # Combine the meaningful word, domain, and 'www.' to create the web URL
    url = f"www.{selected_prefix}{selected_suffix}{selected_domain}"

    return url


# Function to generate a random meaningful company name
def generate_company_name():
    """
    Generate a random meaningful company name.

    Returns:
    - str: Randomly generated company name.
    """
    # Randomly select a prefix and a suffix
    selected_prefix = random.choice(company_prefix)
    selected_suffix = random.choice(company_suffix)

    # Combine the prefix and suffix to create the company name
    company_name = f"{selected_prefix.capitalize()} {selected_suffix.capitalize()}"

    return company_name


# Function to generate a random department
def generate_department(customdept=None, default_departments=default_departments):
    """
    Generate a random department name.

    Parameters:
    - customdept (list, optional): List of custom department names. Default is None.
    - default_departments (list, optional): List of default department names. Default is default_departments.

    Returns:
    - str: Randomly generated department name.
    """
    if customdept:
        return random.choice(customdept)
    else:
        # Default list of departments
        return random.choice(default_departments)


# Function to generate a random designation
def generate_designation(customdes=None, department=None):
    """
    Generate a random job designation.

    Parameters:
    - customdes (list, optional): List of custom job designations. Default is None.
    - department (str, optional): Department name to determine relevant designations. Default is None.

    Returns:
    - str: Randomly generated job designation.
    """
    if customdes:
        return random.choice(customdes)
    elif department:
        relevant_designations = designation_mapping.get(department, [])
        if relevant_designations:
            return random.choice(relevant_designations)
    # If no customdes or relevant designations found, return a default value
    return "Employee"


# Function to generate a random passport number based on country
def generate_passport_number(country_code="US"):
    """
    Generate a random passport number based on the specified country code.

    Parameters:
    - country_code (str, optional): Country code. Default is "US".

    Returns:
    - str: Randomly generated passport number.
    """
    # Check if the country code has a specific format defined
    if country_code in passport_formats:
        format_string = passport_formats[country_code]
    else:
        # Use a default format if no specific format is defined
        format_string = "AA######"

    # Generate a random passport number based on the format
    passport_number = ""
    for char in format_string:
        if char == "A":
            passport_number += random.choice("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
        elif char == "#":
            passport_number += random.choice("0123456789")
        else:
            passport_number += char  # Keep non-placeholder characters as they are

    return passport_number


# Function to generate a random SSN based on country
def generate_ssn(country_code="US"):
    """
    Generate a random Social Security Number (SSN) based on the specified country code.

    Parameters:
    - country_code (str, optional): Country code. Default is "US".

    Returns:
    - str: Randomly generated SSN.
    """
    # Check if the country code has a specific format defined
    if country_code in ssn_formats:
        format_string = ssn_formats[country_code]
    else:
        # Use a default format if no specific format is defined
        format_string = "###-##-####"

    # Generate a random SSN based on the format
    ssn = ""
    for char in format_string:
        if char == "#":
            ssn += random.choice("0123456789")
        else:
            ssn += char  # Keep non-placeholder characters as they are

    return ssn


# Function to generate a random credit card number
def generate_credit_card():
    """
    Generate a random credit card number (simplified example).

    Returns:
    - str: Randomly generated credit card number.
    """
    # Generate a random credit card number (this is a simplified example)
    card_number = "".join(random.choice("0123456789") for _ in range(16))
    return card_number


# Function to generate a random password
def generate_password():
    """
    Generate a random password.

    Returns:
    - str: Randomly generated password.
    """
    # Randomly select words from both lists
    word1 = random.choice(password_words_1)
    word2 = random.choice(password_words_2)

    # Combine the words
    password = f"{word1}{word2}"

    # Add uppercase and lowercase letters to the password
    password += random.choice(string.ascii_uppercase)
    password += random.choice(string.ascii_lowercase)

    # Generate additional random characters to reach a length between 8 and 16
    while len(password) < 8:
        password += random.choice(string.ascii_letters + string.digits)

    return password


# Function to generate a random address based on the country
def generate_address(country="US"):
    """
    Generate a random address based on the specified country.

    Parameters:
    - country (str, optional): Country code. Default is "US".

    Returns:
    - str: Randomly generated address.
    """
    # Get the states/counties and cities for the chosen country
    states_and_cities = country_to_states_and_cities.get(country, {})

    if not states_and_cities:
        return "Invalid country"

    state = random.choice(list(states_and_cities.keys()))
    cities = states_and_cities[state]
    city = random.choice(cities)

    if country == "US":
        postcode = f"{random.randint(10000, 99999)}"
    elif country == "UK":
        postcode = f"{random.choice(['AB', 'AL', 'B', 'BA', 'BB', 'BD', 'BH', 'BL', 'BN', 'BR', 'BS', 'BT', 'CA', 'CB', 'CF', 'CH', 'CM', 'CO', 'CR', 'CT', 'CV', 'CW', 'DA', 'DD', 'DE', 'DG', 'DH', 'DL', 'DN', 'DT', 'DY', 'E', 'EC', 'EH', 'EN', 'EX', 'FK', 'FY', 'G', 'GL', 'GY', 'GU', 'HA', 'HD', 'HG', 'HP', 'HR', 'HS', 'HU', 'HX', 'IG', 'IM', 'IP', 'IV', 'JE', 'KA', 'KT', 'KW', 'KY', 'L', 'LA', 'LD', 'LE', 'LL', 'LN', 'LS', 'LU', 'M', 'ME', 'MK', 'ML', 'N', 'NE', 'NG', 'NN', 'NP', 'NR', 'NW', 'OL', 'OX', 'PA', 'PE', 'PH', 'PL', 'PO', 'PR', 'RG', 'RH', 'RM', 'S', 'SA', 'SE', 'SG', 'SK', 'SL', 'SM', 'SN', 'SO', 'SP', 'SR', 'SS', 'ST', 'SW', 'SY', 'TA', 'TD', 'TF', 'TN', 'TQ', 'TR', 'TS', 'TW', 'UB', 'W','WA', 'WC', 'WD', 'WF', 'WN', 'WR', 'WS', 'WV', 'YO', 'ZE'])}{random.choice(['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16'])} {random.choice(['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16'])}{random.choice('ABCDEFGHIJKLMNOPQRSTUVWXYZ')}{random.choice('ABCDEFGHIJKLMNOPQRSTUVWXYZ')}"
    # Generate a fake street name and house/apartment number
    street = f"{random.choice(['Main','First','Park','Maple','Elm','Oak','High','Broadway','Market','Church','Washington','Central','Pine','Chestnut','Grand','Cedar','Pleasant','Lake','River'])} {random.choice(['Rd.', 'St.', 'Ave.', 'Blvd.', 'Sq.', 'Drive', 'Sqr.'])}"
    house_number = random.randint(1, 100)

    # Combine the address components
    address = f"{house_number} {street}, {city}, {state}, {postcode}, {country}"

    return address


# Function to generate a random latitude based on the country
def generate_latitude(country="US"):
    """
    Generate a random latitude based on the specified country.

    Parameters:
    - country (str, optional): Country code. Default is "US".

    Returns:
    - float: Randomly generated latitude.
    """
    # Get the latitude range for the specified country, default to 'US'
    latitude_range = country_latitudes.get(country, country_latitudes["US"])

    # Generate a random latitude within the specified range
    latitude = round(random.uniform(*latitude_range), 6)

    return latitude


# Function to generate a random longitude based on the country
def generate_longitude(country="US"):
    """
    Generate a random longitude based on the specified country.

    Parameters:
    - country (str, optional): Country code. Default is "US".

    Returns:
    - float: Randomly generated longitude.
    """
    # Get the longitude range for the specified country, default to 'US'
    longitude_range = country_longitudes.get(country, country_longitudes["US"])

    # Generate a random longitude within the specified range
    longitude = round(random.uniform(*longitude_range), 6)

    return longitude


# Function to generate a custom sequential column
def generate_custom_sequence(start=1, step=1, num_rows=100):
    """
    Generate a custom sequential column.

    Parameters:
    - start (int, optional): Starting value of the sequence. Default is 1.
    - step (int, optional): Step between each value in the sequence. Default is 1.
    - num_rows (int, optional): Number of rows in the sequence. Default is 100.

    Returns:
    - list: List of sequential values.
    """
    counter = count(start=start, step=step)
    return [next(counter) for _ in range(num_rows)]


# Function to generate a custom industry
def generate_industry(custom_industry=None):
    """
    Generate a random industry.

    Parameters:
    - custom_industry (list, optional): List of custom industry names. Default is None.

    Returns:
    - str: Randomly generated industry name.
    """
    if custom_industry:
        return random.choice(custom_industry)
    else:
        return random.choice(default_industry)


# Function to generate custom time
def generate_custom_time(start, end, num_rows, clock_type=24):
    """
    Generate custom time values.

    Parameters:
    - start (str): Start time in the format "HH:MM:SS".
    - end (str): End time in the format "HH:MM:SS".
    - num_rows (int): Number of time values to generate.
    - clock_type (int, optional): Type of clock (12 or 24). Default is 24.

    Returns:
    - list: List of randomly generated time values.
    """
    data = []
    if clock_type == 12:
        time_format = "%I:%M:%S %p"  # 12-hour clock format with AM/PM
    elif clock_type == 24:
        time_format = "%H:%M:%S"  # 24-hour clock format
    else:
        raise ValueError("Invalid clock_type. Use 12 or 24.")

    # Convert start and end times to datetime objects
    start_time = datetime.strptime(start, "%H:%M:%S")
    end_time = datetime.strptime(end, "%H:%M:%S")

    # Calculate the time difference in seconds
    time_difference = (end_time - start_time).total_seconds()

    # Generate random times within the specified range
    for _ in range(num_rows):
        random_seconds = random.uniform(0, time_difference)
        new_time = start_time + timedelta(seconds=random_seconds)
        data.append(new_time.strftime(time_format))

    return data


# Function to generate custom years
def generate_custom_year(start, end, num_rows):
    """
    Generate custom years.

    Parameters:
    - start (int): Start year.
    - end (int): End year.
    - num_rows (int): Number of years to generate.

    Returns:
    - list: List of randomly generated years.
    """
    data = []

    # Generate random years within the specified range
    for _ in range(num_rows):
        year = random.randint(start, end)
        data.append(year)

    return data


# Function to generate custom months
def generate_custom_month(start, end, num_rows):
    """
    Generate custom month names.

    Parameters:
    - start (int): Start month.
    - end (int): End month.
    - num_rows (int): Number of months to generate.

    Returns:
    - list: List of randomly generated month names.
    """
    months = [
        "January",
        "February",
        "March",
        "April",
        "May",
        "June",
        "July",
        "August",
        "September",
        "October",
        "November",
        "December",
    ]

    data = []

    # Generate random month names
    for _ in range(num_rows):
        month = random.randint(start, end)
        data.append(months[month - 1])

    return data


# Function to generate custom product categories
def generate_custom_prodcats(custom_categories=None):
    """
    Generate custom product categories.

    Parameters:
    - custom_categories (list, optional): List of custom product categories. Default is None.

    Returns:
    - str: Randomly generated product category.
    """
    if custom_categories is None:
        # Use the default product categories if custom_categories is not provided
        return random.choice(default_product_categories)
    else:
        # Use custom_categories if provided
        return random.choice(custom_categories)


# Function to generate product names based on the selected category
def generate_product_name(category):
    """
    Generate a product name based on the selected category.

    Parameters:
    - category (str): Product category.

    Returns:
    - str: Randomly generated product name.
    """
    if category in product_names_dict:
        return random.choice(product_names_dict[category])
    else:
        # If category is not found, choose randomly from all product names
        all_product_names = [
            name for names in product_names_dict.values() for name in names
        ]
        return random.choice(all_product_names)
